package generaldemo;

import com.adbs.ast.*;
import com.adbs.metadata.*;
import com.adbs.syntax.*;
import com.adbs.querybuilder.*;
import generaldemo.connection_dialogs.*;
import java.awt.Cursor;
import java.awt.Frame;
import java.awt.KeyboardFocusManager;
import java.awt.event.KeyEvent;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Locale;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.Preferences;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.table.DefaultTableModel;
import javax.swing.text.Document;
import javax.swing.text.StyledEditorKit;

public class MainFrame extends javax.swing.JFrame
{
	private static final long serialVersionUID = 1L;

	private PlainTextSQLBuilder plainTextSQLBuilder1;
	
	public MainFrame()
	{
        initComponents();

		/// Set up the QueryBuilder ///////////////////////////////////////////////

		queryBuilder1.beginUpdate();

		try
		{
			// prepare icons for QueryBuilder's trees and dialogs

			Icon folderIcon = createImageIcon("images/folder.png");
			Icon tableIcon = createImageIcon("images/table.png");
			Icon viewIcon = createImageIcon("images/view.png");
			Icon synonymIcon = createImageIcon("images/synonym.png");
			Icon procedureIcon = createImageIcon("images/procedure.png");
			Icon chartIcon = createImageIcon("images/chart_organisation.png");
			Icon folderWithBulletIcon = createImageIcon("images/folder_bullet_green.png");
			Icon folderWithTableIcon = createImageIcon("images/folder_table.png");
			Icon databaseTableIcon = createImageIcon("images/database_table.png");
			Icon bulletIcon = createImageIcon("images/bullet_green.png");

			// set up the Query Structure Tree (left tree)
			
			QueryStructureTreeOptions qsto = queryBuilder1.getQueryStructureTreeOptions();

			qsto.beginUpdate();

			qsto.setFieldIcon(bulletIcon);
			qsto.setFieldsIcon(folderWithBulletIcon);
			qsto.setFromIcon(folderWithTableIcon);
			qsto.setFromObjIcon(databaseTableIcon);
			qsto.setQueriesIcon(chartIcon);

			qsto.endUpdate();

			// set up the Metadata Tree (right tree)

			MetadataTreeOptions mto = queryBuilder1.getMetadataTreeOptions();

			mto.beginUpdate();

			mto.setGroupingType(ObjectsGroupingType.DatabaseSchemaType);
			mto.setSortingType(ObjectsSortingType.SchemaTypeName);

			mto.setDatabaseIcon(folderIcon);
			mto.setSchemaIcon(folderIcon);
			mto.setTablesNodeIcon(folderIcon);
			mto.setViewsNodeIcon(folderIcon);
			mto.setProceduresNodeIcon(folderIcon);
			mto.setSynonymsNodeIcon(folderIcon);
			mto.setSystemTablesIcon(tableIcon);
			mto.setSystemViewsIcon(viewIcon);
			mto.setSystemProceduresIcon(procedureIcon);
			mto.setSystemSynonymsIcon(synonymIcon);
			mto.setUserTablesIcon(tableIcon);
			mto.setUserViewsIcon(viewIcon);
			mto.setUserProceduresIcon(procedureIcon);
			mto.setUserSynonymsIcon(synonymIcon);
			mto.setShowSystemTables(false);
			mto.setShowSystemViews(false);
			mto.setShowSystemProcedures(false);
			mto.setShowSystemSynonyms(false);

			mto.endUpdate();

			// set up the Add Object Dialog

			AddObjectDialogOptions aodo = queryBuilder1.getAddObjectDialogOptions();

			aodo.beginUpdate();

			aodo.setTablesIcon(tableIcon);
			aodo.setViewsIcon(viewIcon);
			aodo.setProceduresIcon(procedureIcon);
			aodo.setSynonymsIcon(synonymIcon);

			aodo.endUpdate();

			// set up the Criteria List control (grid under the diagram pane)

			CriteriaListOptions clo = queryBuilder1.getCriteriaListOptions();
			clo.setExpressionColumnWidth(200);
		}
		catch (Exception ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
		finally
		{
			queryBuilder1.endUpdate();
		}

		// Set up the PlainTextSQLBuilder.
		// (PlainTextSQLBuilder is a component that converts plain text
		// from the output of the QueryBuilder to formatted text).
		
		plainTextSQLBuilder1 = new PlainTextSQLBuilder();
		plainTextSQLBuilder1.setKeywordFormat(KeywordFormat.UpperCase);
		// bind the PlainTextSQLBuilder to the instance of the QueryBuilder.
		plainTextSQLBuilder1.setQueryBuilder(queryBuilder1);

		// handle the notification from PlainTextSQLBuilder about the query text has been changed
		plainTextSQLBuilder1.addSQLUpdatedEventListener(new SQLUpdatedEventListener()
		{
			@Override
			public void sqlUpdatedEventOccurred(SQLUpdatedEvent event)
			{
				try
				{
					jTextPane1.setText(plainTextSQLBuilder1.getSQL());
				}
				catch (QueryBuilderException ex)
				{
					Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
				}
			}
		});


		/// Setup other frame components //////////////////////////////////////////////////////

		// setup coloring for the jTextPane1
		jTextPane1.setEditorKit(new StyledEditorKit()
		{
			@Override
			public Document createDefaultDocument()
			{
				return new SimpleTextHighlighter();
			}
		});
		
		// restore window position

		Preferences prefs = Preferences.userNodeForPackage(MainFrame.class);

		int x = prefs.getInt("x", 0);
		int y = prefs.getInt("y", 0);
		int width = prefs.getInt("width", 0);
		int height = prefs.getInt("height", 0);

		if (width != 0 && height != 0)
		{
			this.setBounds(x, y, width, height);
		}
		else
		{
			// center window on the desktop
			setLocationRelativeTo(null);
		}

		jSplitPane1.setBorder(BorderFactory.createEmptyBorder());
		jSplitPane1.setDividerLocation(prefs.getInt("splitter", 450));

		// force the jTextPane1 to move the focus on the Tab key:

		KeyStroke tab = KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0);
		HashSet keystrokes = new HashSet();
		keystrokes.add(tab);
		jTextPane1.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, keystrokes);

		KeyStroke shiftTab = KeyStroke.getKeyStroke(KeyEvent.VK_TAB, KeyEvent.SHIFT_DOWN_MASK);
		keystrokes = new HashSet();
		keystrokes.add(shiftTab);
		jTextPane1.setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS, keystrokes);
    }

	private void FillTableFromResultSet(final ResultSet resultSet)
	{
		try
		{
			ResultSetMetaData metaData = resultSet.getMetaData();

			int columnCount = metaData.getColumnCount();

			Vector columns = new Vector(columnCount);

			//store column names
			for (int i = 1; i <= columnCount; i++)
			{
				columns.add(metaData.getColumnName(i));
			}

			Vector data = new Vector();
			Vector row;

			//store row data
			while (resultSet.next())
			{
				row = new Vector(columnCount);

				for (int i = 1; i <= columnCount; i++)
				{
					row.add(resultSet.getString(i));
				}

				data.add(row);
			}

			jTable1.setModel(new DefaultTableModel(data, columns));
		}
		catch (SQLException ex)
		{
			Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
	}

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTabbedPane1 = new javax.swing.JTabbedPane();
        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();
        queryBuilder1 = new com.adbs.querybuilder.QueryBuilder();
        dataPanel = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenuItemAccess = new javax.swing.JMenuItem();
        jMenuItemDB2 = new javax.swing.JMenuItem();
        jMenuItemFirebird = new javax.swing.JMenuItem();
        jMenuItemInformix = new javax.swing.JMenuItem();
        jMenuItemMSSQL = new javax.swing.JMenuItem();
        jMenuItemMySQL = new javax.swing.JMenuItem();
        jMenuItemOracle = new javax.swing.JMenuItem();
        jMenuItemPostgreSQL = new javax.swing.JMenuItem();
        jMenuItemSQLite = new javax.swing.JMenuItem();
        jMenu3 = new javax.swing.JMenu();
        jMenuItemRefreshMetadata = new javax.swing.JMenuItem();
        jMenuItemClearMetadata = new javax.swing.JMenuItem();
        menuProperties = new javax.swing.JMenu();
        menuItemProperties = new javax.swing.JMenuItem();
        jMenuHelp = new javax.swing.JMenu();
        jMenuItemAbout = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("General Demo");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        getContentPane().setLayout(new javax.swing.BoxLayout(getContentPane(), javax.swing.BoxLayout.LINE_AXIS));

        jTabbedPane1.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jTabbedPane1StateChanged(evt);
            }
        });

        jSplitPane1.setDividerLocation(450);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane1.setResizeWeight(1.0);

        jTextPane1.setFont(new java.awt.Font("Monospaced", 0, 12));
        jTextPane1.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextPane1FocusLost(evt);
            }
        });
        jScrollPane1.setViewportView(jTextPane1);

        jSplitPane1.setRightComponent(jScrollPane1);
        jSplitPane1.setLeftComponent(queryBuilder1);

        jTabbedPane1.addTab("SQL", jSplitPane1);

        dataPanel.setLayout(new javax.swing.BoxLayout(dataPanel, javax.swing.BoxLayout.LINE_AXIS));

        jTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane2.setViewportView(jTable1);

        dataPanel.add(jScrollPane2);

        jTabbedPane1.addTab("Data", dataPanel);

        getContentPane().add(jTabbedPane1);

        jMenu1.setText("Connect");
        jMenu1.setFont(new java.awt.Font("Segoe UI", 1, 12)); // NOI18N

        jMenuItemAccess.setText("Access");
        jMenuItemAccess.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemAccessActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItemAccess);

        jMenuItemDB2.setText("DB2");
        jMenuItemDB2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemDB2ActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItemDB2);

        jMenuItemFirebird.setText("Firebird");
        jMenuItemFirebird.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemFirebirdActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItemFirebird);

        jMenuItemInformix.setText("Informix");
        jMenuItemInformix.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemInformixActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItemInformix);

        jMenuItemMSSQL.setText("MSSQL");
        jMenuItemMSSQL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemMSSQLActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItemMSSQL);

        jMenuItemMySQL.setText("MySQL");
        jMenuItemMySQL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemMySQLActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItemMySQL);

        jMenuItemOracle.setText("Oracle");
        jMenuItemOracle.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemOracleActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItemOracle);

        jMenuItemPostgreSQL.setText("PostgreSQL");
        jMenuItemPostgreSQL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemPostgreSQLActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItemPostgreSQL);

        jMenuItemSQLite.setText("SQLite");
        jMenuItemSQLite.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemSQLiteActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItemSQLite);

        jMenuBar1.add(jMenu1);

        jMenu3.setText("Metadata");

        jMenuItemRefreshMetadata.setText("Refresh Metadata");
        jMenuItemRefreshMetadata.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemRefreshMetadataActionPerformed(evt);
            }
        });
        jMenu3.add(jMenuItemRefreshMetadata);

        jMenuItemClearMetadata.setText("Clear Metadata");
        jMenuItemClearMetadata.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemClearMetadataActionPerformed(evt);
            }
        });
        jMenu3.add(jMenuItemClearMetadata);

        jMenuBar1.add(jMenu3);

        menuProperties.setText("Options");

        menuItemProperties.setText("Query Builder Properties...");
        menuItemProperties.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuItemPropertiesActionPerformed(evt);
            }
        });
        menuProperties.add(menuItemProperties);

        jMenuBar1.add(menuProperties);

        jMenuHelp.setText("About");

        jMenuItemAbout.setText("About...");
        jMenuItemAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItemAboutActionPerformed(evt);
            }
        });
        jMenuHelp.add(jMenuItemAbout);

        jMenuBar1.add(jMenuHelp);

        setJMenuBar(jMenuBar1);

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void jMenuItemAccessActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemAccessActionPerformed
	{//GEN-HEADEREND:event_jMenuItemAccessActionPerformed

		AccessConnectionDialog dlg = new AccessConnectionDialog();

		if (dlg.showDialog())
		{
			try
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				JdbcMetadataProvider metadataProvider = new JdbcMetadataProvider();

				Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
				metadataProvider.setConnection(DriverManager.getConnection(dlg.url, dlg.user, dlg.password));

				UniversalSyntaxProvider syntaxProvider = new UniversalSyntaxProvider();

				queryBuilder1.setSyntaxProvider(syntaxProvider);
				queryBuilder1.setMetadataProvider(metadataProvider);

				queryBuilder1.refreshMetadata();
			}
			catch (ClassNotFoundException ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, "Failed to load driver: \n" + ex.getMessage());
			}
			catch (Exception ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			finally
			{
				setCursor(Cursor.getDefaultCursor());
			}
		}

		dlg.dispose();
}//GEN-LAST:event_jMenuItemAccessActionPerformed

	private void jMenuItemDB2ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemDB2ActionPerformed
	{//GEN-HEADEREND:event_jMenuItemDB2ActionPerformed
		DB2ConnectionDialog dlg = new DB2ConnectionDialog();
		
		if (dlg.showDialog())
		{
			try
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				JdbcMetadataProvider metadataProvider = new JdbcMetadataProvider();
				com.ibm.db2.jcc.DB2SimpleDataSource ds = new com.ibm.db2.jcc.DB2SimpleDataSource();

				ds.setServerName(dlg.server);
				ds.setPortNumber(dlg.port);
				ds.setDatabaseName(dlg.database);
				ds.setDriverType(4);

				metadataProvider.setConnection(ds.getConnection(dlg.user, dlg.password));

				UniversalSyntaxProvider syntaxProvider = new UniversalSyntaxProvider();

				queryBuilder1.setSyntaxProvider(syntaxProvider);
				queryBuilder1.setMetadataProvider(metadataProvider);

				queryBuilder1.refreshMetadata();
			}
			catch (Exception ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			finally
			{
				setCursor(Cursor.getDefaultCursor());
			}
		}

		dlg.dispose();
}//GEN-LAST:event_jMenuItemDB2ActionPerformed

	private void jMenuItemFirebirdActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemFirebirdActionPerformed
	{//GEN-HEADEREND:event_jMenuItemFirebirdActionPerformed
		
		FirebirdConnectionDialog dlg = new FirebirdConnectionDialog();
		
		if (dlg.showDialog())
		{
			try
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				JdbcMetadataProvider metadataProvider = new JdbcMetadataProvider();

				Class.forName("org.firebirdsql.jdbc.FBDriver");
				metadataProvider.setConnection(DriverManager.getConnection(dlg.url, dlg.user, dlg.password));

				UniversalSyntaxProvider syntaxProvider = new UniversalSyntaxProvider();

				queryBuilder1.setSyntaxProvider(syntaxProvider);
				queryBuilder1.setMetadataProvider(metadataProvider);

				queryBuilder1.refreshMetadata();
			}
			catch (ClassNotFoundException ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, "Failed to load driver: \n" + ex.getMessage());
			}
			catch (Exception ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			finally
			{
				setCursor(Cursor.getDefaultCursor());
			}
		}

		dlg.dispose();
}//GEN-LAST:event_jMenuItemFirebirdActionPerformed

	private void jMenuItemInformixActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemInformixActionPerformed
	{//GEN-HEADEREND:event_jMenuItemInformixActionPerformed
		InformixConnectionDialog dlg = new InformixConnectionDialog();

		if (dlg.showDialog())
		{
			try
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				JdbcMetadataProvider metadataProvider = new JdbcMetadataProvider();

				Class.forName("com.informix.jdbc.IfxDriver");
				metadataProvider.setConnection(DriverManager.getConnection(dlg.url, dlg.user, dlg.password));

				UniversalSyntaxProvider syntaxProvider = new UniversalSyntaxProvider();

				queryBuilder1.setSyntaxProvider(syntaxProvider);
				queryBuilder1.setMetadataProvider(metadataProvider);

				queryBuilder1.refreshMetadata();
			}
			catch (ClassNotFoundException ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, "Failed to load driver: \n" + ex.getMessage());
			}
			catch (Exception ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			finally
			{
				setCursor(Cursor.getDefaultCursor());
			}
		}

		dlg.dispose();
}//GEN-LAST:event_jMenuItemInformixActionPerformed

	private void jMenuItemMSSQLActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemMSSQLActionPerformed
	{//GEN-HEADEREND:event_jMenuItemMSSQLActionPerformed
		MSSQLConnectionDialog dlg = new MSSQLConnectionDialog();

		if (dlg.showDialog())
		{
			try
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				JdbcMetadataProvider metadataProvider = new JdbcMetadataProvider();
				Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
				metadataProvider.setConnection(DriverManager.getConnection(dlg.url, dlg.user, dlg.password));

				queryBuilder1.setSyntaxProvider(new UniversalSyntaxProvider());
				queryBuilder1.setMetadataProvider(metadataProvider);

				queryBuilder1.refreshMetadata();
			}
			catch (ClassNotFoundException ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, "Failed to load driver: \n" + ex.getMessage());
			}
			catch (Exception ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			finally
			{
				setCursor(Cursor.getDefaultCursor());
			}
		}

		dlg.dispose();
}//GEN-LAST:event_jMenuItemMSSQLActionPerformed

	private void jMenuItemMySQLActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemMySQLActionPerformed
	{//GEN-HEADEREND:event_jMenuItemMySQLActionPerformed
		MySQLConnectionDialog dlg = new MySQLConnectionDialog();

		if (dlg.showDialog())
		{
			try
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				JdbcMetadataProvider metadataProvider = new JdbcMetadataProvider();
				Class.forName("com.mysql.jdbc.Driver");
				metadataProvider.setConnection(DriverManager.getConnection(dlg.url, dlg.user, dlg.password));

				UniversalSyntaxProvider syntaxProvider = new UniversalSyntaxProvider();

				queryBuilder1.setSyntaxProvider(syntaxProvider);
				queryBuilder1.setMetadataProvider(metadataProvider);

				queryBuilder1.refreshMetadata();
			}
			catch (ClassNotFoundException ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, "Failed to load driver: \n" + ex.getMessage());
			}
			catch (Exception ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			finally
			{
				setCursor(Cursor.getDefaultCursor());
			}
		}

		dlg.dispose();
}//GEN-LAST:event_jMenuItemMySQLActionPerformed

	private void jMenuItemOracleActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemOracleActionPerformed
	{//GEN-HEADEREND:event_jMenuItemOracleActionPerformed
		OracleConnectionDialog dlg = new OracleConnectionDialog();
		
		if (dlg.showDialog())
		{
			try
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				JdbcMetadataProvider metadataProvider = new JdbcMetadataProvider();
				Class.forName("oracle.jdbc.driver.OracleDriver");

				Locale originalLocale = Locale.getDefault();

				try
				{
					Locale.setDefault(Locale.US);

					metadataProvider.setConnection(DriverManager.getConnection(dlg.url, dlg.user, dlg.password));
				}
				finally
				{
					Locale.setDefault(originalLocale);
				}

				UniversalSyntaxProvider syntaxProvider = new UniversalSyntaxProvider();

				queryBuilder1.setSyntaxProvider(syntaxProvider);
				queryBuilder1.setMetadataProvider(metadataProvider);

				queryBuilder1.refreshMetadata();
			}
			catch (ClassNotFoundException ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, "Failed to load driver: \n" + ex.getMessage());
			}
			catch (Exception ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			finally
			{
				setCursor(Cursor.getDefaultCursor());
			}
		}

		dlg.dispose();
}//GEN-LAST:event_jMenuItemOracleActionPerformed

	private void jMenuItemPostgreSQLActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemPostgreSQLActionPerformed
	{//GEN-HEADEREND:event_jMenuItemPostgreSQLActionPerformed
		PostgreSQLConnectionDialog dlg = new PostgreSQLConnectionDialog();
		
		if (dlg.showDialog())
		{
			try
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				JdbcMetadataProvider metadataProvider = new JdbcMetadataProvider();

				Class.forName("org.postgresql.Driver");
				metadataProvider.setConnection(DriverManager.getConnection(dlg.url, dlg.user, dlg.password));

				UniversalSyntaxProvider syntaxProvider = new UniversalSyntaxProvider();

				queryBuilder1.setSyntaxProvider(syntaxProvider);
				queryBuilder1.setMetadataProvider(metadataProvider);

				queryBuilder1.refreshMetadata();
			}
			catch (ClassNotFoundException ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, "Failed to load driver: \n" + ex.getMessage());
			}
			catch (Exception ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			finally
			{
				setCursor(Cursor.getDefaultCursor());
			}
		}

		dlg.dispose();
}//GEN-LAST:event_jMenuItemPostgreSQLActionPerformed

	private void jMenuItemSQLiteActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemSQLiteActionPerformed
	{//GEN-HEADEREND:event_jMenuItemSQLiteActionPerformed
		SQLiteConnectionDialog dlg = new SQLiteConnectionDialog();
		
		if (dlg.showDialog())
		{
			try
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				JdbcMetadataProvider metadataProvider = new JdbcMetadataProvider();

				Class.forName("org.sqlite.JDBC");
				metadataProvider.setConnection(DriverManager.getConnection(dlg.url));

				UniversalSyntaxProvider syntaxProvider = new UniversalSyntaxProvider();

				queryBuilder1.setSyntaxProvider(syntaxProvider);
				queryBuilder1.setMetadataProvider(metadataProvider);

				queryBuilder1.refreshMetadata();
			}
			catch (ClassNotFoundException ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, "Failed to load driver: \n" + ex.getMessage());
			}
			catch (Exception ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			finally
			{
				setCursor(Cursor.getDefaultCursor());
			}
		}

		dlg.dispose();
}//GEN-LAST:event_jMenuItemSQLiteActionPerformed

	private void jMenuItemRefreshMetadataActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemRefreshMetadataActionPerformed
	{//GEN-HEADEREND:event_jMenuItemRefreshMetadataActionPerformed
		if (queryBuilder1.getMetadataProvider() != null && queryBuilder1.getMetadataProvider().isConnected())
		{
			queryBuilder1.refreshMetadata();
		}
}//GEN-LAST:event_jMenuItemRefreshMetadataActionPerformed

	private void jMenuItemClearMetadataActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemClearMetadataActionPerformed
	{//GEN-HEADEREND:event_jMenuItemClearMetadataActionPerformed
		queryBuilder1.getMetadataContainer().clear();
}//GEN-LAST:event_jMenuItemClearMetadataActionPerformed

	private void menuItemPropertiesActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_menuItemPropertiesActionPerformed
	{//GEN-HEADEREND:event_menuItemPropertiesActionPerformed
		queryBuilder1.showQueryBuilderPropertiesDialog(this, "Active Query Builder Properties", EnumSet.allOf(QueryBuilderProperties.class),
				plainTextSQLBuilder1);
}//GEN-LAST:event_menuItemPropertiesActionPerformed

	private void jMenuItemAboutActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jMenuItemAboutActionPerformed
	{//GEN-HEADEREND:event_jMenuItemAboutActionPerformed
		AboutDialog dlg = new AboutDialog(this, true);
		dlg.setVisible(true);
}//GEN-LAST:event_jMenuItemAboutActionPerformed

	private void jTextPane1FocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_jTextPane1FocusLost
	{//GEN-HEADEREND:event_jTextPane1FocusLost
		// when the focus is going out of the text pane, update the query builder with new query text
		try
		{
			queryBuilder1.setSQL(jTextPane1.getText());
		}
		catch (QueryBuilderException ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
}//GEN-LAST:event_jTextPane1FocusLost

	private void formWindowClosing(java.awt.event.WindowEvent evt)//GEN-FIRST:event_formWindowClosing
	{//GEN-HEADEREND:event_formWindowClosing
		// save window position
		if (this.getExtendedState() == Frame.NORMAL)
		{
			Preferences prefs = Preferences.userNodeForPackage(MainFrame.class);

			prefs.putInt("x", this.getX());
			prefs.putInt("y", this.getY());
			prefs.putInt("width", this.getWidth());
			prefs.putInt("height", this.getHeight());
			prefs.putInt("splitter", jSplitPane1.getDividerLocation());
		}
	}//GEN-LAST:event_formWindowClosing

	private void jTabbedPane1StateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_jTabbedPane1StateChanged
	{//GEN-HEADEREND:event_jTabbedPane1StateChanged
		if (jTabbedPane1.getSelectedComponent() == dataPanel)
		{
			// clear JTable
			jTable1.setModel(new DefaultTableModel(new Vector(), new Vector()));

			// Update the query text in the query builder.
			// this because the StateChanged event of the TabbedPane arrives before the JTextPane1 lost the focus
			// and updates the query text.
			try
			{
				queryBuilder1.setSQL(jTextPane1.getText());
			}
			catch (QueryBuilderException ex)
			{
				Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
				JOptionPane.showMessageDialog(this, ex.getMessage());
				return;
			}

			// execute query:

			JdbcMetadataProvider metadataProvider = (JdbcMetadataProvider) queryBuilder1.getMetadataProvider();

			if (queryBuilder1.getMetadataProvider() != null && queryBuilder1.getMetadataProvider().isConnected())
			{
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				try
				{
					ResultSet resultSet = metadataProvider.getConnection().createStatement().executeQuery(queryBuilder1.getSQL());

					// fill JTable
					FillTableFromResultSet(resultSet);
				}
				catch (SQLException ex)
				{
					Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
					JOptionPane.showMessageDialog(this, ex.getMessage());
				}
				finally
				{
					setCursor(Cursor.getDefaultCursor());
				}
			}
		}
	}//GEN-LAST:event_jTabbedPane1StateChanged

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel dataPanel;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu3;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenu jMenuHelp;
    private javax.swing.JMenuItem jMenuItemAbout;
    private javax.swing.JMenuItem jMenuItemAccess;
    private javax.swing.JMenuItem jMenuItemClearMetadata;
    private javax.swing.JMenuItem jMenuItemDB2;
    private javax.swing.JMenuItem jMenuItemFirebird;
    private javax.swing.JMenuItem jMenuItemInformix;
    private javax.swing.JMenuItem jMenuItemMSSQL;
    private javax.swing.JMenuItem jMenuItemMySQL;
    private javax.swing.JMenuItem jMenuItemOracle;
    private javax.swing.JMenuItem jMenuItemPostgreSQL;
    private javax.swing.JMenuItem jMenuItemRefreshMetadata;
    private javax.swing.JMenuItem jMenuItemSQLite;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTable jTable1;
    private javax.swing.JTextPane jTextPane1;
    private javax.swing.JMenuItem menuItemProperties;
    private javax.swing.JMenu menuProperties;
    private com.adbs.querybuilder.QueryBuilder queryBuilder1;
    // End of variables declaration//GEN-END:variables

	protected Icon createImageIcon(String path)
	{
		java.net.URL imgURL = MainFrame.class.getResource(path);

		if (imgURL != null)
		{
			return new ImageIcon(imgURL);
		}
		else
		{
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	// convert java.sql.Types value to string
	public static String typeToString(int sqlType)
	{
		switch (sqlType)
		{
			case Types.ARRAY: return "ARRAY";
			case Types.BIGINT: return "BIGINT";
			case Types.BINARY: return "BINARY";
			case Types.BIT: return "BIT";
			case Types.BLOB: return "BLOB";
			case Types.BOOLEAN: return "BOOLEAN";
			case Types.CHAR: return "CHAR";
			case Types.CLOB: return "CLOB";
			case Types.DATALINK: return "DATALINK";
			case Types.DATE: return "DATE";
			case Types.DECIMAL: return "DECIMAL";
			case Types.DISTINCT: return "DISTINCT";
			case Types.DOUBLE: return "DOUBLE";
			case Types.FLOAT: return "FLOAT";
			case Types.INTEGER: return "INTEGER";
			case Types.JAVA_OBJECT: return "JAVA_OBJECT";
			case Types.LONGNVARCHAR: return "LONGNVARCHAR";
			case Types.LONGVARBINARY: return "LONGVARBINARY";
			case Types.LONGVARCHAR: return "LONGVARCHAR";
			case Types.NCHAR: return "NCHAR";
			case Types.NCLOB: return "NCLOB";
			case Types.NULL: return "NULL";
			case Types.NUMERIC: return "NUMERIC";
			case Types.NVARCHAR: return "NVARCHAR";
			case Types.OTHER: return "OTHER";
			case Types.REAL: return "REAL";
			case Types.REF: return "REF";
			case Types.ROWID: return "ROWID";
			case Types.SMALLINT: return "SMALLINT";
			case Types.SQLXML: return "SQLXML";
			case Types.STRUCT: return "STRUCT";
			case Types.TIME: return "TIME";
			case Types.TIMESTAMP: return "TIMESTAMP";
			case Types.TINYINT: return "TINYINT";
			case Types.VARBINARY: return "VARBINARY";
			case Types.VARCHAR: return "VARCHAR";
			default: return "UNKNOWN";
		}
	}
}
