package generaldemo;

import javax.swing.text.*;
import java.awt.*;
import java.util.*;
import java.util.regex.*;

public class SimpleTextHighlighter extends DefaultStyledDocument
{
	private static final long serialVersionUID = 1L;
	
	private Element rootElement;
	private HashMap<String, Color> keywords = new HashMap<String, Color>();
	private MutableAttributeSet style;
	private Color commentColor = Color.red;
	private Pattern singleLineCommentDelimter = Pattern.compile("--");
	private Pattern multiLineCommentDelimiterStart = Pattern.compile("/\\*");
	private Pattern multiLineCommentDelimiterEnd = Pattern.compile("\\*/");

	public SimpleTextHighlighter()
	{
		putProperty(DefaultEditorKit.EndOfLineStringProperty, "\n");

		rootElement = getDefaultRootElement();
		style = new SimpleAttributeSet();

		keywords.put("select", Color.blue);
		keywords.put("from", Color.blue);
		keywords.put("where", Color.blue);
		keywords.put("order", Color.blue);
		keywords.put("group", Color.blue);
		keywords.put("by", Color.blue);
		keywords.put("union", Color.blue);
		keywords.put("inner", Color.blue);
		keywords.put("outer", Color.blue);
		keywords.put("join", Color.blue);
		keywords.put("distinct", Color.blue);
		keywords.put("all", Color.blue);
		keywords.put("with", Color.blue);
		keywords.put("case", Color.blue);
		keywords.put("when", Color.blue);
		keywords.put("then", Color.blue);
		keywords.put("like", Color.blue);
		keywords.put("in", Color.blue);
		keywords.put("and", Color.blue);
		keywords.put("or", Color.blue);
		keywords.put("begin", Color.blue);
		keywords.put("end", Color.blue);
		keywords.put("between", Color.blue);
		keywords.put("break", Color.blue);
		keywords.put("continue", Color.blue);
		keywords.put("desc", Color.blue);
		keywords.put("asc", Color.blue);
		keywords.put("if", Color.blue);
		keywords.put("else", Color.blue);
		keywords.put("exec", Color.blue);
		keywords.put("execute", Color.blue);
		keywords.put("exists", Color.blue);
		keywords.put("for", Color.blue);
		keywords.put("goto", Color.blue);
		keywords.put("having", Color.blue);
		keywords.put("top", Color.blue);
		keywords.put("as", Color.blue);
		keywords.put("on", Color.blue);
		keywords.put("left", Color.blue);
		keywords.put("right", Color.blue);
		keywords.put("full", Color.blue);
	}

	@Override
	public void insertString(int offset, String str, AttributeSet attr) throws BadLocationException
	{
		super.insertString(offset, str, attr);

		processChangedLines(offset, str.length());
	}

	@Override
	public void remove(int offset, int length) throws BadLocationException
	{
		super.remove(offset, length);
		
		processChangedLines(offset, length);
	}

	public void processChangedLines(int offset, int length) throws BadLocationException
	{
		String text = getText(0, getLength());
		highlightString(Color.black, 0, getLength(), true, false);

		Set<String> keyw = keywords.keySet();

		for (String keyword : keyw)
		{
			Color col = keywords.get(keyword);

			Pattern p = Pattern.compile("\\b" + keyword + "\\b", Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(text);

			while (m.find())
			{
				highlightString(col, m.start(), keyword.length(), true, true);
			}
		}

		Matcher mlcStart = multiLineCommentDelimiterStart.matcher(text);
		Matcher mlcEnd = multiLineCommentDelimiterEnd.matcher(text);

		while (mlcStart.find())
		{
			if (mlcEnd.find(mlcStart.end()))
			{
				highlightString(commentColor, mlcStart.start(), (mlcEnd.end() - mlcStart.start()), true, true);
			}
			else
			{
				highlightString(commentColor, mlcStart.start(), getLength(), true, true);
			}
		}

		Matcher slc = singleLineCommentDelimter.matcher(text);

		while (slc.find())
		{
			int line = rootElement.getElementIndex(slc.start());
			int endOffset = rootElement.getElement(line).getEndOffset() - 1;

			highlightString(commentColor, slc.start(), (endOffset - slc.start()), true, true);
		}
	}

	public void highlightString(Color col, int begin, int length, boolean flag, boolean bold)
	{
		StyleConstants.setForeground(style, col);
		StyleConstants.setBold(style, bold);

		setCharacterAttributes(begin, length, style, flag);
	}
}
